package com.huimv.apiservice.controller;

import com.huimv.apiservice.entity.SleepStatusEntity;
import com.huimv.apiservice.entity.vo.*;
import com.huimv.apiservice.limit.annotation.Limit;
import com.huimv.apiservice.service.PigService;
import com.huimv.common.utils.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.CrossOrigin;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.List;
import java.util.Map;
import java.util.Objects;

/**
 * @author yinhao
 * @date 2021/5/8 16:09
 */
@CrossOrigin
@RestController
@RequestMapping("/pig")
public class PigController {

    @Autowired
    private PigService pigService;

    /**
     * 获取猪基本信息
     * @param accessToken token
     * @param pigEarTagNo 耳标号
     * @return
     */
    @Limit(key = "pig_getPigInfo",name = "pig/getPigInfo",prefix = "apiservice")
    @RequestMapping("/getPigInfo")
    public R getPigInfo(@RequestParam(value = "accessToken", required = false) String accessToken,
                        @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验
        Map<String,Object> map = pigService.getPigInfoByEarTagNo(pigEarTagNo);
        return Objects.requireNonNull(R.ok().put("code", 1000)).put("data", map);
    }


    /**
     * 获取多媒体接口(照片/视频)
     * @param accessToken token
     * @param pigEarTagNo 耳标号
     * @return
     */
    @RequestMapping("/getImage")
    public R getImage(@RequestParam(value = "accessToken", required = false) String accessToken,
                      @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验
        PigImageVo pigImageVo = pigService.getImageByEarTagNo(pigEarTagNo);
        return Objects.requireNonNull(R.ok().put("code", 1000)).put("data", pigImageVo);
    }

    @RequestMapping("/getListByBreed")
    public R getListByBreed(@RequestParam(value = "accessToken", required = false) String accessToken,
                            @RequestParam("breedName") String breedName) {
        //TODO token校验
        List<PigBaseVo> pigBaseVoList = pigService.getListByBreed(breedName);
        return Objects.requireNonNull(R.ok().put("code", 1000)).put("data", pigBaseVoList);
    }

    @RequestMapping("/getPigPedigree")
    public R getPigPedigree(@RequestParam(value = "accessToken", required = false) String accessToken,
                            @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验
        PigPedigreeVo pigPedigreeVo = pigService.getPigPedigree(pigEarTagNo);
        return Objects.requireNonNull(R.ok().put("code", 1000)).put("data", pigPedigreeVo);
    }

    @RequestMapping("/getKeeper")
    public R getKeeper(@RequestParam(value = "accessToken", required = false) String accessToken,
                       @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验
        PigKeeperVo pigKeeperVo = pigService.getKeeper(pigEarTagNo);
        return Objects.requireNonNull(R.ok().put("code", 1000)).put("data", pigKeeperVo);
    }

    @RequestMapping("/getIndoorEnv")
    public R getIndoorEnv(@RequestParam(value = "accessToken", required = false) String accessToken,
                       @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验
        EnvVo envVo = pigService.getIndoorEnv(pigEarTagNo);
        return Objects.requireNonNull(R.ok().put("code", 1000)).put("data", envVo);
    }

    @RequestMapping("/getFeedTimes")
    public R getFeedTimes(@RequestParam(value = "accessToken", required = false) String accessToken,
                          @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验

//          = pigService.getFeedTimes(pigEarTagNo);
//        return Objects.requireNonNull(R.ok().put("code", 1000)).put("data", envVo);
        return null;
    }

    //8.获取当天的睡眠状态数据接⼝
    @RequestMapping("/getSleepStatus")
    public R getSleepStatus(@RequestParam(value = "accessToken", required = false) String accessToken,
                               @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验
        SleepStatusEntity sleepStatusEntity = pigService.getSleepStatus(pigEarTagNo);



        return Objects.requireNonNull(Objects.requireNonNull(R.ok().put("code", 1000)).put("msg", "查询成功")).put("data",sleepStatusEntity );
    }

    //9.获取当天的运动数据接⼝
    @RequestMapping("/getActivityAmount")
    public R getActivityAmount(@RequestParam(value = "accessToken", required = false) String accessToken,
                       @RequestParam("pigEarTagNo") String pigEarTagNo) {
        //TODO token校验
        PigSportVo pigSportVo = pigService.getActivityAmount(pigEarTagNo);


        return Objects.requireNonNull(Objects.requireNonNull(R.ok().put("code", 1000)).put("msg", "查询成功")).put("data",pigSportVo );
    }

}
